from src.models.trading_model import TradingModel
from src.training import train_model
from src.utils import save_model, load_model

class ExampleAgent:
    def __init__(self, config):
        """
        Initializes the ExampleAgent with a configuration dictionary.

        Parameters:
        config (dict): The configuration dictionary containing the parameters for agent and models.
        """
        self.transformer_config = config['transformer']
        self.rl_config = config['rl']
        self.transformer_model = TransformerModel(self.transformer_config)
        self.rl_model = RLModel(self.rl_config)
        self.trading_agent = TradingModel(self.transformer_model, self.rl_model)

    def train(self, training_data):
        """
        Trains the agent on the provided training data.

        Parameters:
        training_data (np.array): The training data for the agent.
        """
        # Training the transformer model
        self.transformer_model = train_model(self.transformer_model, training_data, self.transformer_config)
        
        # Training the RL model
        self.rl_model = train_model(self.rl_model, training_data, self.rl_config)
        
        # Update the trading_agent with the trained models
        self.trading_agent = TradingModel(self.transformer_model, self.rl_model)

    def act(self, obs):
        """
        Makes a decision based on the provided observation.

        Parameters:
        obs (np.array): The observation for the agent.

        Returns:
        int: The action chosen by the agent (0: hold, 1: buy, 2: sell).
        """
        return self.trading_agent.make_decision(obs)

    def save_models(self, path):
        """
        Saves the models to the specified path.

        Parameters:
        path (str): The path to save the models.
        """
        # Save the transformer model
        save_model(self.transformer_model, path + "/transformer_model.pth")
        
        # Save the RL model
        save_model(self.rl_model, path + "/rl_model.pth")

    def load_models(self, path):
        """
        Loads the models from the specified path.

        Parameters:
        path (str): The path to load the models from.
        """
        # Load the transformer model
        self.transformer_model = load_model(TransformerModel(self.transformer_config), path + "/transformer_model.pth")
        
        # Load the RL model
        self.rl_model = load_model(RLModel(self.rl_config), path + "/rl_model.pth")
        
        # Update the trading_agent with the loaded models
        self.trading_agent = TradingModel(self.transformer_model, self.rl_model)
