import torch
from torch.utils.data import DataLoader
from src.models.transformer_model import TransformerModel
from src.models.rl_model import RLModel
from src.data.data_preprocessing import Dataset
from sklearn.metrics import accuracy_score

def evaluate_transformer_model(transformer_model, test_data):
    """
    Evaluate the Transformer model.

    Parameters:
    transformer_model (TransformerModel): The transformer model to evaluate.
    test_data (Dataset): The test data.
    """
    # Create data loader
    dataloader = DataLoader(test_data, batch_size=32, shuffle=False)

    # Set device
    device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
    transformer_model.to(device)

    # Define loss function
    criterion = torch.nn.CrossEntropyLoss()

    # Evaluation mode
    transformer_model.eval()

    # Evaluation loop
    total_loss = 0
    total_correct = 0
    total_samples = 0
    with torch.no_grad():
        for i, (inputs, targets) in enumerate(dataloader):
            inputs, targets = inputs.to(device), targets.to(device)

            # Forward pass
            outputs = transformer_model(inputs)
            
            # Compute loss
            loss = criterion(outputs, targets)
            total_loss += loss.item()

            # Compute accuracy
            _, predicted = torch.max(outputs, 1)
            total_correct += (predicted == targets).sum().item()
            total_samples += targets.size(0)

        # Compute average loss and accuracy
        average_loss = total_loss / len(dataloader)
        accuracy = total_correct / total_samples
        print(f'Average loss: {average_loss}, Accuracy: {accuracy}')

def evaluate_rl_model(rl_model, env, episodes):
    """
    Evaluate the RL model.

    Parameters:
    rl_model (RLModel): The RL model to evaluate.
    env (gym.Env): The Gym environment to use for evaluation.
    episodes (int): The number of episodes to evaluate for.
    """
    total_rewards = []
    for i_episode in range(episodes):
        state = env.reset()
        done = False
        episode_reward = 0
        while not done:
            action = rl_model.predict(state)
            state, reward, done, _ = env.step(action)
            episode_reward += reward
        total_rewards.append(episode_reward)
        print(f'Episode: {i_episode+1}, Total reward: {episode_reward}')

    # Compute average reward
    average_reward = sum(total_rewards) / episodes
    print(f'Average total reward: {average_reward}')
