from src.data import data_collection, data_preprocessing
from src.models import transformer_model, rl_model, trading_model
from src.training import train
from src.evaluation import evaluate
from src.utils import utils, metrics

def main(config):
    """
    The main function that drives the training and evaluation of the models.

    Parameters:
    config (dict): The configuration dictionary.
    """
    try:
        # Set seed for reproducibility
        utils.seed_everything(config['seed'])

        # Data Collection - Retrieve the data from the specified source
        raw_data = data_collection.collect_data(config['data_source'])

        # Data Preprocessing - Process the raw data for the models
        processed_data = data_preprocessing.preprocess_data(raw_data)

        # Model Creation - Create the Transformer, RL, and Trading models
        transformer = transformer_model.TransformerModel(config['transformer_config'])
        rl_agent = rl_model.RLModel(config['rl_config'])
        trading_agent = trading_model.TradingAgent(transformer, rl_agent)

        # Model Training - Train the trading agent on the processed data
        train.train(trading_agent, processed_data, config['training_config'])

        # Model Evaluation - Evaluate the trained agent and print the results
        evaluation_results = evaluate.evaluate(trading_agent, processed_data, metrics)

        print(evaluation_results)
    except Exception as e:
        print(f"An error occurred during the execution: {e}")

if __name__ == "__main__":
    try:
        # Configuration dictionary
        config = {
            'seed': 42,
            'data_source': 'data/raw/',
            'transformer_config': 'config/transformer.json',
            'rl_config': 'config/rl.json',
            'training_config': 'config/training.json'
        }

        main(config)
    except Exception as e:
        print(f"An error occurred during the setup or during main execution: {e}")
