from stable_baselines3 import PPO, A2C, DDPG
from stable_baselines3.common.envs import DummyVecEnv

class RLModel:
    def __init__(self, env, algorithm='PPO'):
        """
        Initializes the RLModel with a given environment and algorithm.

        Parameters:
        env (gym.Env): The Gym environment to use for training.
        algorithm (str): The RL algorithm to use for training. Should be one of ['PPO', 'A2C', 'DDPG'].
        """
        self.env = DummyVecEnv([lambda: env])  # The environment must be vectorized
        if algorithm == 'PPO':
            self.model = PPO('MlpPolicy', self.env, verbose=1)
        elif algorithm == 'A2C':
            self.model = A2C('MlpPolicy', self.env, verbose=1)
        elif algorithm == 'DDPG':
            self.model = DDPG('MlpPolicy', self.env, verbose=1)
        else:
            raise ValueError(f'Unknown algorithm {algorithm}. Should be one of ["PPO", "A2C", "DDPG"]')

    def train(self, timesteps):
        """
        Trains the model for a given number of timesteps.

        Parameters:
        timesteps (int): The number of timesteps to train for.
        """
        self.model.learn(total_timesteps=timesteps)

    def predict(self, obs):
        """
        Makes a prediction based on the given observations.

        Parameters:
        obs (np.array): The observations to base the prediction on.

        Returns:
        np.array: The action predicted by the model.
        """
        action, _states = self.model.predict(obs)
        return action

    def save(self, path):
        """
        Saves the model at the given path.

        Parameters:
        path (str): The path to save the model at.
        """
        self.model.save(path)

    def load(self, path):
        """
        Loads the model from the given path.

        Parameters:
        path (str): The path to load the model from.
        """
        self.model = self.model.load(path)
