from transformers import AutoModel, AutoTokenizer

class TransformerModel:
    def __init__(self, pretrained_model_name='bert-base-uncased', max_length=512, mode='text'):
        self.tokenizer = AutoTokenizer.from_pretrained(pretrained_model_name)
        self.model = AutoModel.from_pretrained(pretrained_model_name)
        self.max_length = max_length
        self.mode = mode

    def preprocess_data(self, data):
        """
        Preprocesses the input data, converting it into tensors that can be fed into the model.
        
        Parameters:
        data (str, list[str], or pd.Series): Data to preprocess. Can be a single string, a list of strings, or a pandas Series of numerical values.
        
        Returns:
        torch.Tensor: Preprocessed tensors ready for model input.
        """
        if self.mode == 'text':
            if isinstance(data, str):
                data = [data]
            inputs = self.tokenizer(data, return_tensors='pt', truncation=True, padding=True, max_length=self.max_length)
        elif self.mode == 'time_series':
            # Ensure data is a pandas Series
            if not isinstance(data, pd.Series):
                data = pd.Series(data)
            inputs = torch.tensor(data.values).float().unsqueeze(0)
        else:
            raise ValueError(f"Invalid mode: {self.mode}. Mode must be either 'text' or 'time_series'.")
        
        return inputs

    def get_embeddings(self, data):
        """
        Returns the embeddings generated by the transformer model.
        
        Parameters:
        data (str, list[str], or pd.Series): Data to get embeddings for. Can be a single string, a list of strings, or a pandas Series of numerical values.
        
        Returns:
        torch.Tensor: Embeddings for the input data.
        """
        inputs = self.preprocess_data(data)
        outputs = self.model(**inputs)
        return outputs.last_hidden_state
